# ./addons/eventscripts/knifeduel/knifeduel.py

from random import choice

import es
import gamethread
from cfglib import AddonCFG
from usermsg import saytext2

class KnifeDuel(object):
    def __init__(self):
        self.delay  = None
        self.scout  = None
        self.time   = None
        self.winner = 0
        self.second = 0
        self.duel   = False
        self.cash   = {}
    def scriptLoad(self):
        config = AddonCFG(es.getAddonPath('knifeduel') + '/knifeduel_config.cfg')
        config.text('****************************************')
        config.text('* KnifeDuel by BackRaw - Configuration *')
        config.text('****************************************')
        config.text('')
        self.delay = config.cvar('knifeduel_delay', 1, 'How many seconds does it take to strip the players off their weapons and give them a knife? (0 = immediately)')
        self.scout = config.cvar('knifeduel_scout', 1, 'Give the winner a free scout at next spawn? (1/0)')
        self.time  = config.cvar('knifeduel_time', 60, 'How many seconds does a KnifeDuel take?')
        config.text('\n')
        config.text('****************************************')
        config.text('* KnifeDuel by BackRaw - Configuration *')
        config.text('****************************************')
        config.write()
        config.execute()
        self.cancel()
        es.ServerVar('knifeduel_ver', 1.0, "BackRaw's KnifeDuel").makepublic()
    def scriptUnload(self):
        self.cancel()
        self.cash.clear()
    def playerSpawn(self, userid):
        if userid in self.cash:
            es.setplayerprop(userid, 'CCSPlayer.m_iAccount', self.cash[userid])
            del self.cash[userid]
        if self.winner and userid == self.winner:
            self.winner = 0
            player      = es.createplayerlist(userid)[userid]
            saytext2('#all', player['index'], '\x03KnifeDuel\x01: You should fear and respect the mighty \x03%s' % player['name'])
            if int(self.scout):
                es.server.queuecmd('es_xgive %s weapon_scout' % userid)
    def playerDeath(self, attacker):
        if not self.duel:
            if es.getlivingplayercount(2) == 1 and es.getlivingplayercount(3) == 1:
                self.duel   = True
                self.second = int(self.time) + 2
                self.duelLoop()
        else:
            self.winner = attacker
            self.cancel()
            es.centermsg('KnifeDuel: %s won!' % es.getplayername(attacker))
    def removePlayer(self, userid):
        if userid in self.cash:
            del self.cash[userid]
    def strip(self, userid):
        es.setplayerprop(userid, 'CBasePlayer.m_iHealth', 100)
        es.server.cmd('es_xfire %s func_buyzone Disable' % userid)
        es.server.cmd('es_xfire %s player_weaponstrip Kill' % userid)
        es.server.cmd('es_xgive %s player_weaponstrip' % userid)
        es.server.cmd('es_xfire %s player_weaponstrip Strip' % userid)
        es.delayed(float(self.delay), 'es_xgive %s item_assaultsuit;es_xgive %s weapon_knife' % (userid, userid))
    def duelLoop(self):
        if self.duel:
            if self.second:
                tell = True
                if self.second == int(self.time) + 2:
                    tell = False
                elif self.second == int(self.time) + 1:
                    tell = False
                    entlist = es.createentitylist()
                    for index in entlist:
                        if entlist[index]['classname'].startswith('weapon_'):
                            es.server.cmd('es_xremove %s' % index)
                    lP = self.livingPlayers
                    u1 = choice(lP)
                    lP.remove(u1)
                    u2 = lP[0]
                    self.strip(u1)
                    self.strip(u2)
                    x, y, z = es.getplayerlocation(u1)
                    z = float(z) + 85
                    es.centertell(u2, "You'll be placed where %s walked a second before." % es.getplayername(u1))
                    es.delayed(1, 'es_xsetpos %s %s %s %s' % (u2, x, y, z))
                if tell:
                    es.centermsg('KnifeDuel: %s' % self.second)
                self.second -= 1
                gamethread.delayedname(1, 'knifeduel_loop', self.duelLoop)
            else:
                self.winner = 0
                self.cancel()
                es.centermsg('KnifeDuel: GAME OVER!')
                for userid in self.livingPlayers:
                    self.cash[userid] = es.getplayerprop(userid, 'CCSPlayer.m_iAccount')
                es.server.cmd('mp_restartgame 1')
        else:
            self.cancel()
    def cancel(self):
        for userid in self.livingPlayers:
            es.server.cmd('es_xfire %s func_buyzone Enable' % userid)
        self.duel = False
        gamethread.cancelDelayed('knifeduel_loop')
    @property
    def livingPlayers(self):
        return filter(lambda x: not es.getplayerprop(x, 'CBasePlayer.pl.deadflag'), es.getUseridList())
kd = KnifeDuel()

def player_spawn(ev):
    if int(ev['es_userteam']) > 1:
        kd.playerSpawn(int(ev['userid']))

def player_death(ev):
    kd.playerDeath(int(ev['attacker']))

def player_disconnect(ev):
    kd.removePlayer(int(ev['userid']))

def round_end(ev):
    kd.cancel()

def load():
    kd.scriptLoad()

def unload():
    kd.scriptUnload()